from collections import Counter
import logging

from ruxit.api.base_plugin import RemoteBasePlugin

from controlm_api import ControlM

logger = logging.getLogger(__name__)


class ControlMPlugin(RemoteBasePlugin):
    def query(self, **kwargs):
        config = kwargs["config"]

        c = ControlM(config["url"], config["user"], config["pass"], logger=logger)

        group = self.topology_builder.create_group(f"Control-M", f"ControlM")
        device = group.create_device(f'Control-M {config["url"]}')

        base_counter = Counter(
            {
                "wait_condition": 0,
                "ended_ok": 0,
                "executing": 0,
                "wait_resource": 0,
                "wait_user": 0,
                "ended_not_ok": 0,
                "unknown": 0,
                "not_in_ajf": 0,
            }
        )

        try:
            with c.login() as token:

                job_status = c.job_status(token)
                logger.info(f"Parsing: {job_status}")

                base_counter.update(Counter(job.status.replace(" ", "_").lower() for job in job_status.jobs))

                for key, value in base_counter.items():
                    device.absolute(key, value)

                for failed_job in [job for job in job_status.jobs if job.status == "Ended Not OK"]:
                    logger.info(f"Found failed job: {failed_job}")
                    title = f"Control-M: Job {failed_job.name} Ended NOT OK"
                    description = f"Job {failed_job.name} has status {failed_job.status}"
                    props = {
                        "Name": failed_job.name,
                        "ID": failed_job.job_id,
                        "Start time": failed_job.start_time.strftime("%d/%m/%Y %H:%M:%S")
                        if failed_job.start_time is not None
                        else "",
                        "End time": failed_job.end_time.strftime("%d/%m/%Y %H:%M:%S") if failed_job.end_time is not None else "",
                        "Status": failed_job.status,
                        "Runs": f"{failed_job.number_of_runs}",
                    }
                    device.report_error_event(description, title, properties=props)
        except Exception as e:
            error = f"ControlM Plugin - Error {e}"
            logger.exception(error)
            device.report_error_event(error, "ControlM Plugin - Error executing")
